/**
 *
 * @file common_magma.h
 *
 *  clMAGMA (version 1.2.0) --
 *  Univ. of Tennessee, Knoxville
 *  Univ. of California, Berkeley
 *  Univ. of Colorado, Denver
 *  @date September 2014
 *
 *
 **/

/***************************************************************************//**
 *  MAGMA facilities of interest to both src and magmablas directories
 **/
#ifndef MAGMA_COMMON_H
#define MAGMA_COMMON_H

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <ctype.h>

#if defined( _WIN32 ) || defined( _WIN64 )

  #include "magmawinthread.h"
  #include <windows.h>
  #include <limits.h>
  #include <io.h>

    // functions where Microsoft fails to provide C99 standard
    // (only with Microsoft, not with nvcc on Windows)
    // in both common_magma.h and testings.h
    #ifndef __NVCC__
    
        #include <float.h>
        #define copysign(x,y) _copysign(x,y)
        #define isnan(x)      _isnan(x)
        #define isinf(x)      ( ! _finite(x) && ! _isnan(x) )
        #define isfinite(x)   _finite(x)
        // note _snprintf has slightly different semantics than snprintf
        #define snprintf _snprintf
    
    #endif

#else

  #include <pthread.h>
  #include <unistd.h>
  #include <inttypes.h>

#endif

// provide our own support for pthread_barrier on MacOS and Windows
#include "pthread_barrier.h"

#include "magma.h"
#include "magma_lapack.h"
//#include "operators.h"
#include "transpose.h"
#include "magma_threadsetting.h"

/** ****************************************************************************
 *  Determine if weak symbols are allowed
 */
#if defined(linux) || defined(__linux) || defined(__linux__)
#if defined(__GNUC_EXCL__) || defined(__GNUC__)
#define MAGMA_HAVE_WEAK    1
#endif
#endif

/***************************************************************************//**
 *  Global utilities
 **/
#ifndef max
#define max(a, b) ((a) > (b) ? (a) : (b))
#endif

#ifndef min
#define min(a, b) ((a) < (b) ? (a) : (b))
#endif

#ifndef roundup
#define roundup(a, b) (b <= 0) ? (a) : (((a) + (b)-1) & ~((b)-1))
#endif

#ifndef ceildiv
#define ceildiv(a, b) ((a - 1)/b + 1)
#endif


/** ****************************************************************************
 *  Define magma_[sd]sqrt functions
 *    - sqrt alone cannot be caught by the generation script because of tsqrt
 */

#define magma_dsqrt sqrt
#define magma_ssqrt sqrtf

/** ****************************************************************************
 *  Define macros for error checking
 */
void chk_helper( int err, const char* func, const char* file, int line );

// if debugging is enabled, assert that all functions return correctly
#ifndef NDEBUG
#  define chk( err ) chk_helper( err, __func__, __FILE__, __LINE__  )
#else
#  define chk( err ) (err)
#endif

#endif /* MAGMA_COMMON_H */
