/*
 *   -- micMAGMA (version 1.2.0) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      April 2012
 *
 *   @author Azzam Haidar
 *   @author Khairul Kabir
 *
 *   @precisions normal z -> s d c
 *
 *   This is included in magma_mic_server.cpp,
 *   inside a switch { } block in the do_work() function.
 */

case magma_mic_ZCOPY:
{
    //printf("zcopy\n");
    magma_mic_zcopy_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZCOPY\n");
        return;
    }

    mic_time = dsecnd();
    call_zcopy(param);
    mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_ZGEMM:
{
    //printf("zgemm\n");
    magma_mic_zgemm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZGEMM\n");
        return;
    }

    mic_time = dsecnd();
    call_zgemm(param);
    mic_time = dsecnd() - mic_time;

    m = param.m;
    n = param.n;
    k = param.k;
    gflops = MICFLOPS_GEMM((double)m, (double)n, (double)k) * 1e-9;
    //printf("zgemm: %d\t%d\t%d\t%8.2lf\t%8.2lf\t%8.2lf\n", m, n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_ZHEMM:
{
    //printf("zhemm\n");
    magma_mic_zhemm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZHEMM\n");
        return;
    }

    mic_time = dsecnd();
    call_zhemm(param);
    mic_time = dsecnd() - mic_time;

    m = param.m;
    n = param.n;
    magma_side_t side = param.side;

    //gflops = MICFLOPS_SYMM((magma_side_t)side, (double)m, (double)n ) * 1e-9;
    //printf("zhemm: %d\t%d\t%g\t%g\t%g\n", m, n, mic_time,gflops, gflops/mic_time);
    break;
}

case magma_mic_ZGEMV:
{
    //printf("zgemv\n");
    magma_mic_zgemv_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZGEMV\n");
        return;
    }

    mic_time = dsecnd();
    call_zgemv(param);
    mic_time = dsecnd() - mic_time;

    m = param.m;
    n = param.n;
    //gflops = MICFLOPS_GEMV((double)m, (double)n) * 1e-9;
    //printf("zgemv: %d\t%d\t%g\t%g\t%g\n", m, n, mic_time,
    //        gflops, gflops/mic_time);
    break;
}

case magma_mic_ZHEMV:
{
    //printf("zhemv\n");
    magma_mic_zhemv_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZHEMV\n");
        return;
    }

    mic_time = dsecnd();
    call_zhemv(param);
    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_ZSYMV2:
{
    //printf("zsymv2\n");
    magma_mic_zsymv2_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZHEMV\n");
        return;
    }

    mic_time = dsecnd();
    call_zsymv2(param);
    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_ZTRSM:
{
    //printf("ztrsm\n");
    magma_mic_ztrsm_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZTRSM\n");
        return;
    }

    mic_time = dsecnd();
    call_ztrsm(param);
    mic_time = dsecnd() - mic_time;

    side = param.side;
    m = param.m;
    n = param.n;

    if (side == MagmaLeft)
        gflops = MICFLOPS_TRSM_L((double)m, (double)n) * 1e-9;
    else
        gflops = MICFLOPS_TRSM_R((double)m, (double)n) * 1e-9;

    //printf("ztrsm: %c\t%d\t%d\t%f\t%f\t%f\n", side, m, n, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_ZTRSV:
{
    //printf("ztrsv\n");

    magma_mic_ztrsv_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZTRSV\n");
        return;
    }
    
    mic_time = dsecnd();
    call_ztrsv(param);
    mic_time = dsecnd() - mic_time;
    
    break;
}

case magma_mic_ZTRMV:
{
    //printf("ztrmv\n");
    
    magma_mic_ztrmv_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZTRMV\n");
        return;
    }
    
    mic_time = dsecnd();
    call_ztrmv(param);
    mic_time = dsecnd() - mic_time;
    
    break;
}

case magma_mic_ZTRMM:
{
    //printf("ztrmm\n");
    magma_mic_ztrmm_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZTRMM\n");
        return;
    }

    mic_time = dsecnd();
    call_ztrmm(param);
    mic_time = dsecnd() - mic_time;

    side = param.side;
    m = param.m;
    n = param.n;

    //if (side == MagmaLeft)
    //    gflops = MICFLOPS_TRSM_L((float)m, (float)n) * 1e-9;
    //else
    //    gflops = MICFLOPS_TRSM_R((float)m, (float)n) * 1e-9;

    //printf("ztrmm: %c\t%d\t%d\t%f\t%f\t%f\n", side, m, n, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_ZHERK:
{
    //printf("zherk\n");
    magma_mic_zherk_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZHERK\n");
        return;
    }

    mic_time = dsecnd();
    call_zherk(param);
    mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYRK((float)n, (float)k) * 1e-9;
    //printf("zherk: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_ZHERK_MMIC:
{
    //printf("zherk\n");
    magma_mic_zherk_mmic_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZHERK_MMIC\n");
        return;
    }

    mic_time = dsecnd();
    call_zherk_mmic(param);
    mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYRK((float)n, (float)k) * 1e-9;
    //printf("zherk: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}


case magma_mic_ZHER2K:
{
    //printf("zher2k\n");
    magma_mic_zher2k_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZHER2K\n");
        return;
    }

    mic_time = dsecnd();
    call_zher2k(param);
    mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYR2K((float)n, (float)k) * 1e-9;
    //printf("zher2k: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_ZHER2K_MMIC:
{
    //printf("zher2k_mmic\n");
    magma_mic_zher2k_mmic_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZHER2K_MMIC\n");
        return;
    }

    //mic_time = dsecnd();
    call_zher2k_mmic(param);
    //mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYRK((float)n, (float)k) * 1e-9;
    //printf("zherk: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_ZLARFT:
{
    //printf("zlarft\n");
    magma_mic_zlarft_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZLARFT\n");
        return;
    }

    //mic_time = dsecnd();
    call_zlarft(param);
    //mic_time = dsecnd() - mic_time;

    //gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("zlarft: %d\t%d\t%d\t%f\t%f\t%f\n", m, n, k, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_ZLARFB:
{
    //printf("zlarfb\n");
    magma_mic_zlarfb_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZLARFB\n");
        return;
    }

    //mic_time = dsecnd();
    call_zlarfb(param);
    //mic_time = dsecnd() - mic_time;

    //gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("zlarfb: %d\t%d\t%d\t%f\t%f\t%f\n", m, n, k, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_ZLARFB_GEMM:
{
    //printf("zlarfb\n");
    magma_mic_zlarfb_gemm_param zlarfb_gemm_param;
    if (mic_recv(&zlarfb_gemm_param, sizeof(zlarfb_gemm_param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZLARFB_GEMM\n");
        return;
    }

    //mic_time = dsecnd();
    call_zlarfb_gemm(zlarfb_gemm_param);
    //mic_time = dsecnd() - mic_time;

    //gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("zlarfb: %d\t%d\t%d\t%f\t%f\t%f\n", m, n, k, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_ZTRSM_GEMM:
{
    //printf("ztrsmgemm\n");
    magma_mic_ztrsmgemm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZTRSM_GEMM\n");
        return;
    }

    mic_time = dsecnd();
    call_ztrsmgemm(param);
    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_ZLASET:
{
    magma_mic_zlaset_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZLASET\n");
        return;
    }

    call_zlaset(param);

    break;
}

case magma_mic_ZLASWP:
{
    magma_mic_zlaswp_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZLASWP\n");
        return;
    }

    call_zlaswp(param);

    break;
}

case magma_mic_ZSWAP:
{
    magma_mic_zswap_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZSWAP\n");
        return;
    }

    call_zswap(param);

    break;
}

case magma_mic_ZTRANSPOSE:
{
    magma_mic_ztranspose_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZTRANSPOSE\n");
        return;
    }

    call_ztranspose(param);
    break;
}
